@@ -53,7 +53,7 @@ static const char copyright[] =
 static char sccsid[] = "@(#)dumpfs.c	8.5 (Berkeley) 4/29/95";
 #endif
 static const char rcsid[] =
-  "$FreeBSD: src/sbin/dumpfs/dumpfs.c,v 1.44.2.1.2.1 2009/04/15 03:14:26 kensmith Exp $";
+  "$FreeBSD: src/sbin/dumpfs/dumpfs.c,v 1.44.2.3.2.1 2010/02/10 00:26:20 kensmith Exp $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -160,7 +160,7 @@ dumpfs(const char *name)
 		fstime = afs.fs_old_time;
 		printf("magic\t%x (UFS1)\ttime\t%s",
 		    afs.fs_magic, ctime(&fstime));
-		printf("id\t[ %x %x ]\n", afs.fs_id[0], afs.fs_id[1]);
+		printf("id\t[ %08x %08x ]\n", afs.fs_id[0], afs.fs_id[1]);
 		printf("ncg\t%d\tsize\t%jd\tblocks\t%jd\n",
 		    afs.fs_ncg, (intmax_t)fssize, (intmax_t)afs.fs_dsize);
 		break;
@@ -413,7 +413,7 @@ marshal(const char *name)
 		break;
 	}
 	/* -p..r unimplemented */
-	printf("-s %jd ", (intmax_t)fs->fs_size);
+	printf("-s %jd ", (intmax_t)fsbtodb(fs, fs->fs_size));
 	printf("%s ", disk.d_name);
 	printf("\n");
 
