@@ -57,7 +57,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)fsck.h	8.4 (Berkeley) 5/9/95
- * $FreeBSD: src/sbin/fsck_ffs/fsck.h,v 1.37.2.4.2.2 2009/04/27 19:15:33 delphij Exp $
+ * $FreeBSD: src/sbin/fsck_ffs/fsck.h,v 1.37.2.6.2.1 2010/02/10 00:26:20 kensmith Exp $
  */
 
 #include <unistd.h>
@@ -297,8 +297,8 @@ int	lfmode;			/* lost & found directory creation mode */
 ufs2_daddr_t n_blks;		/* number of blocks in use */
 ino_t n_files;			/* number of files in use */
 
-int	got_siginfo;		/* received a SIGINFO */
-int	got_sigalarm;		/* received a SIGALRM */
+volatile sig_atomic_t	got_siginfo;	/* received a SIGINFO */
+volatile sig_atomic_t	got_sigalarm;	/* received a SIGALRM */
 
 #define	clearinode(dp) \
 	if (sblock.fs_magic == FS_UFS1_MAGIC) { \
