@@ -52,7 +52,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sbin/fsck_ffs/gjournal.c,v 1.2.8.1 2009/04/15 03:14:26 kensmith Exp $");
+__FBSDID("$FreeBSD: src/sbin/fsck_ffs/gjournal.c,v 1.2.2.1.2.1 2010/02/10 00:26:20 kensmith Exp $");
 
 #include <sys/param.h>
 #include <sys/disklabel.h>
@@ -672,7 +672,7 @@ gjournal_check(const char *filesys)
 
 	devnam = filesys;
 	getdisk();
-	/* Are there any unreferenced inodes in this cylinder group? */
+	/* Are there any unreferenced inodes in this file system? */
 	if (fs->fs_unrefs == 0) {
 		//printf("No unreferenced inodes.\n");
 		closedisk();
