@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/lib/libufs/block.c,v 1.10.30.1 2009/04/15 03:14:26 kensmith Exp $");
+__FBSDID("$FreeBSD: src/lib/libufs/block.c,v 1.10.20.1.2.1 2010/02/10 00:26:20 kensmith Exp $");
 
 #include <sys/param.h>
 #include <sys/mount.h>
@@ -63,8 +63,10 @@ bread(struct uufsd *disk, ufs2_daddr_t blockno, void *data, size_t size)
 	 */
 	if (((intptr_t)data) & 0x3f) {
 		p2 = malloc(size);
-		if (p2 == NULL)
+		if (p2 == NULL) {
 			ERROR(disk, "allocate bounce buffer");
+			goto fail;
+		}
 	}
 	cnt = pread(disk->d_fd, p2, size, (off_t)(blockno * disk->d_bsize));
 	if (cnt == -1) {
@@ -114,8 +116,10 @@ bwrite(struct uufsd *disk, ufs2_daddr_t blockno, const void *data, size_t size)
 	 */
 	if (((intptr_t)data) & 0x3f) {
 		p2 = malloc(size);
-		if (p2 == NULL)
+		if (p2 == NULL) {
 			ERROR(disk, "allocate bounce buffer");
+			return (-1);
+		}
 		memcpy(p2, data, size);
 		data = p2;
 	}
